<?php

namespace Amp\Parallel;

class PanicError extends \Error {
    /** @var string Class name of uncaught exception. */
    private $name;

    /** @var string Stack trace of the panic. */
    private $trace;

    /**
     * Creates a new panic error.
     *
     * @param string $name The uncaught exception class.
     * @param string $message The panic message.
     * @param mixed  $code The panic code.
     * @param string $trace The panic stack trace.
     */
    public function __construct(string $name, string $message = '', $code = 0, string $trace = '') {
        parent::__construct($message);

        // Code might also be a string, because PDO, so don't pass to parent constructor, but assign directly.
        // See https://github.com/amphp/parallel/issues/19.
        $this->code = $code;
        $this->name = $name;
        $this->trace = $trace;
    }

    /**
     * Returns the class name of the uncaught exception.
     *
     * @return string
     */
    public function getName(): string {
        return $this->name;
    }

    /**
     * Gets the stack trace at the point the panic occurred.
     *
     * @return string
     */
    public function getPanicTrace(): string {
        return $this->trace;
    }
}
